@@ -158,10 +158,13 @@ class cGAN():
             input_domain,
             target_domain,
             epochs,
+            batch_size,
+            len_dataset,
             save_ckpt=5,
             verbose=1,
             max_outputs=2,
-            initial_step=0):
+            initial_step=0,
+            mlflow=False):
         """Trains both models for n EPOCHS. Saves ckpts every n EPOCHS.
         The training loop together with the optimization algorithm define the
         learning algorithm.
@@ -184,22 +187,33 @@ class cGAN():
         """
         if self.generator_optimizer is None:
             raise ModelNotCompiledError("use conGAN.compile() first.")
+        if mlflow:
+            import mlflow
 
         # Retrace workaround @function signature only tensors.
         step = tf.Variable(initial_step, name="step", dtype=tf.int64)
 
-        num_batches = tf.data.experimental.cardinality(input_domain).numpy()
+        num_batches = len_dataset / batch_size
         # Keras Progbar
         progbar = tf.keras.utils.Progbar(target=num_batches, verbose=verbose)
         file_writer = tf.summary.create_file_writer(self.tensorboard)
         file_writer.set_as_default()
+
+        # Iterate over all epochs.
         for epoch in range(epochs):
             step_float = 0
             start = time.time()
             for input_images, target_images in zip(input_domain,
                                                    target_domain):
-                self.train_step(input_images, target_images, step, max_outputs,
-                                file_writer)
+                generated_images, source_images, target_images = self.train_step(
+                    input_images, target_images, step, max_outputs,
+                    file_writer)
+                # TODO Workaround. summary image & gpu usage
+                with file_writer.as_default():
+                    tf.summary.image("generated", generated_images, step=step)
+                    tf.summary.image("source", input_images, step=step)
+                    tf.summary.image("target", target_images, step=step)
+
                 file_writer.flush()
                 progbar.update(current=(step_float))
                 step_float += 1
@@ -260,7 +274,7 @@ class cGAN():
             self.discriminator.save(os.path.join(model_dir, "discriminator"))
 
     @tf.function
-    def train_step(self, input_images, target_images, step, max_outputs,
+    def train_step(self, source_images, target_images, step, max_outputs,
                    file_writer):
         """Decorated function (@tf.function) that creates a callable tensorflow
         graph from a python function.
@@ -270,30 +284,19 @@ class cGAN():
             # Record gradients for generator and discriminator for each seperate
             # training step.
             with tf.GradientTape() as gen_tape, tf.GradientTape() as disc_tape:
-                generated_images = self.generator(input_images, training=True)
-                tf.summary.image("generated_images",
-                                 generated_images,
-                                 max_outputs=max_outputs,
-                                 step=step)
-                tf.summary.image("target_images",
-                                 target_images,
-                                 max_outputs=max_outputs,
-                                 step=step)
-                tf.summary.image("input_images",
-                                 input_images,
-                                 max_outputs=max_outputs,
-                                 step=step)
+                generated_images = self.generator(source_images, training=True)
+                disc_target = self.discriminator(
+                    [source_images, target_images], training=True)
+                disc_generated = self.discriminator(
+                    [source_images, generated_images], training=True)
 
-                real_output = self.discriminator([input_images, target_images],
-                                                 training=True)
-                fake_output = self.discriminator(
-                    [input_images, generated_images], training=True)
-
-                gen_loss = self.generator_loss(fake_output, generated_images,
-                                               target_images, self.l1_weight)
+                gen_loss = self.generator_loss(disc_generated,
+                                               generated_images, target_images,
+                                               self.l1_weight)
                 tf.summary.scalar("generator_loss", gen_loss, step=step)
 
-                disc_loss = self.discriminator_loss(real_output, fake_output)
+                disc_loss = self.discriminator_loss(disc_target,
+                                                    disc_generated)
                 tf.summary.scalar("discriminator_loss", disc_loss, step=step)
 
             gradients_of_generator = gen_tape.gradient(
@@ -325,6 +328,8 @@ class cGAN():
             self.discriminator_optimizer.apply_gradients(
                 zip(gradients_of_discriminator,
                     self.discriminator.trainable_variables))
+            # Piece of shit workaround.
+            return generated_images, source_images, target_images
 
     @staticmethod
     def discriminator_loss(real_output, fake_output):
