@@ -128,10 +128,13 @@ class CycleGAN(object):
             source_domain,
             target_domain,
             epochs,
+            batch_size,
+            len_dataset,
             save_ckpt=5,
             verbose=1,
             max_outputs=2,
-            initial_step=0):
+            initial_step=0,
+            mlflow=False):
         """Trains all four models for n EPOCHS. Saves ckpts every n EPOCHS.
         The training loop together with the optimization algorithm define the
         learning algorithm.
@@ -155,22 +158,36 @@ class CycleGAN(object):
         """
         if self.generator_target_optimizer is None:
             raise ModelNotCompiledError("use CycleGAN.compile() first.")
+        if mlflow:
+            import mlflow
 
         # Retrace workaround @function signature only tensors.
         step = tf.Variable(initial_step, name="step", dtype=tf.int64)
 
-        num_batches = tf.data.experimental.cardinality(source_domain).numpy()
+        num_batches = len_dataset / batch_size
         # Keras Progbar
         progbar = tf.keras.utils.Progbar(target=num_batches, verbose=verbose)
         file_writer = tf.summary.create_file_writer(self.tensorboard)
         file_writer.set_as_default()
+
+        # Iterate over all epochs.
         for epoch in range(epochs):
             step_float = 0
             start = time.time()
             for source_images, target_images in zip(source_domain,
                                                     target_domain):
-                self.train_step(source_images, target_images, step,
-                                max_outputs, file_writer)
+                imgs = self.train_step(source_images, target_images, step,
+                                       max_outputs, file_writer)
+
+                # TODO Workaround. summary image & gpu usage
+                with file_writer.as_default():
+                    tf.summary.image("source", imgs[0], step=step)
+                    tf.summary.image("target", imgs[1], step=step)
+                    tf.summary.image("generated_source", imgs[2], step=step)
+                    tf.summary.image("generated_target", imgs[3], step=step)
+                    tf.summary.image("cycled_source", imgs[4], step=step)
+                    tf.summary.image("cycled_target", imgs[5], step=step)
+
                 file_writer.flush()
                 progbar.update(current=(step_float))
                 step_float += 1
@@ -251,39 +268,16 @@ class CycleGAN(object):
             # Record gradients for generator and discriminator for each seperate
             # training step.
             with tf.GradientTape(persistent=True) as tape:
-                tf.summary.image("target_images",
-                                 target_images,
-                                 step=step,
-                                 max_outputs=max_outputs)
-                tf.summary.image("source_images",
-                                 source_images,
-                                 step=step,
-                                 max_outputs=max_outputs)
                 generated_target_images = self.generator_target(source_images,
                                                                 training=True)
-                tf.summary.image("generated_target_images",
-                                 generated_target_images,
-                                 step=step,
-                                 max_outputs=max_outputs)
                 cycled_source_images = self.generator_source(
                     generated_target_images, training=True)
-                tf.summary.image("cycled_source_images",
-                                 cycled_source_images,
-                                 step=step,
-                                 max_outputs=max_outputs)
 
                 generated_source_images = self.generator_source(target_images,
                                                                 training=True)
-                tf.summary.image("generated_source_images",
-                                 generated_source_images,
-                                 step=step,
-                                 max_outputs=max_outputs)
+
                 cycled_target_images = self.generator_target(
                     generated_source_images, training=True)
-                tf.summary.image("cycled_target_images",
-                                 cycled_target_images,
-                                 step=step,
-                                 max_outputs=max_outputs)
 
                 # same_source and same_target are used for identity loss.
                 same_source = self.generator_source(target_images,
@@ -362,6 +356,14 @@ class CycleGAN(object):
                 zip(gradients_of_discriminator_source,
                     self.discriminator_source.trainable_variables))
 
+            imgs = [
+                source_images, target_images, generated_source_images,
+                generated_target_images, cycled_source_images,
+                cycled_target_images
+            ]
+            # Piece of shit workaround.
+            return imgs
+
     @staticmethod
     def discriminator_loss(real, generated):
         """This method quantifies how well the discriminator is able to
