@@ -253,6 +253,7 @@ class GAN():
             dataset,
             epochs,
             batch_size,
+            len_dataset,
             save_ckpt=5,
             verbose=1,
             max_outputs=2,
@@ -286,17 +287,24 @@ class GAN():
         # Retrace workaround @function signature only tensors.
         step = tf.Variable(initial_step, name="step", dtype=tf.int64)
 
-        num_batches = tf.data.experimental.cardinality(dataset).numpy()
+        num_batches = len_dataset / batch_size
         # Keras Progbar
         progbar = tf.keras.utils.Progbar(target=num_batches, verbose=verbose)
         file_writer = tf.summary.create_file_writer(self.tensorboard)
         file_writer.set_as_default()
+
+        # Iterate over all epochs.
         for epoch in range(epochs):
             step_float = 0
             start = time.time()
             for image_batch in dataset:
-                self.train_step(image_batch, batch_size, step, max_outputs,
-                                file_writer)
+                generated_images, target_images = self.train_step(
+                    image_batch, batch_size, step, max_outputs, file_writer)
+                # TODO Workaround. summary image & gpu usage
+                with file_writer.as_default():
+                    tf.summary.image("generated", generated_images, step=step)
+                    tf.summary.image("target", target_images, step=step)
+
                 file_writer.flush()
                 progbar.update(current=(step_float))
                 step_float += 1
@@ -359,7 +367,7 @@ class GAN():
             self.discriminator.save(os.path.join(model_dir, "discriminator"))
 
     @tf.function
-    def train_step(self, real_images, batch_size, step, max_outputs,
+    def train_step(self, target_images, batch_size, step, max_outputs,
                    file_writer):
         """Decorated function (@tf.function) that creates a callable tensorflow
         graph from a python function.
@@ -371,23 +379,15 @@ class GAN():
             # training step.
             with tf.GradientTape() as gen_tape, tf.GradientTape() as disc_tape:
                 generated_images = self.generator(z_latent, training=True)
-                tf.summary.image("generated_images",
-                                 generated_images,
-                                 max_outputs=max_outputs,
-                                 step=step)
-                tf.summary.image("real_images",
-                                 real_images,
-                                 max_outputs=max_outputs,
-                                 step=step)
+                disc_target = self.discriminator(target_images, training=True)
+                disc_generated = self.discriminator(generated_images,
+                                                    training=True)
 
-                real_output = self.discriminator(real_images, training=True)
-                fake_output = self.discriminator(generated_images,
-                                                 training=True)
-
-                gen_loss = self.generator_loss(fake_output)
+                gen_loss = self.generator_loss(disc_generated)
                 tf.summary.scalar("generator_loss", gen_loss, step=step)
 
-                disc_loss = self.discriminator_loss(real_output, fake_output)
+                disc_loss = self.discriminator_loss(disc_target,
+                                                    disc_generated)
                 tf.summary.scalar("discriminator_loss", disc_loss, step=step)
 
             gradients_of_generator = gen_tape.gradient(
@@ -419,6 +419,8 @@ class GAN():
             self.discriminator_optimizer.apply_gradients(
                 zip(gradients_of_discriminator,
                     self.discriminator.trainable_variables))
+            # Piece of shit workaround.
+            return generated_images, target_images
 
     @staticmethod
     def discriminator_loss(real_output, fake_output):
