@@ -13,10 +13,6 @@
 # limitations under the License.
 # =============================================================================
 """DnnLab metric learning API"""
-#from dnnlab.losses.yolo import yolo_loss
-from dnnlab.metric_learning.triplet_net import Encoder
-from dnnlab.metric_learning.triplet_net import TripletNet 
+from dnnlab.metric_learning.triplet_net import Encoder,TripletNet, Metrics
 from dnnlab.metric_learning.similarity_attention import SimilarityAttention
-from dnnlab.metric_learning.triplet_net import Metrics
-
-from dnnlab.metric_learning import utils
\ No newline at end of file
+from dnnlab.metric_learning import triplet_utils
\ No newline at end of file
