@@ -20,16 +20,15 @@ import time
 from datetime import datetime
 import numpy as np
 import io
-
 import tensorflow as tf 
+import matplotlib.pyplot as plt
 
 from dnnlab.errors.dnnlab_exceptions import ModelNotCompiledError
 from dnnlab.losses.triplet_net.triplet_loss import TripletBatchHardLoss, SimilarityLoss, OverallSimilarityLoss
 from dnnlab.metric_learning.similarity_attention import SimilarityAttention
+from dnnlab.metric_learning.triplet_utils import construct_mined_triplets
 
-import dnnlab.metric_learning.utils as utils
 
-import matplotlib.pyplot as plt
 
 
 os.environ["TF_CPP_MIN_LOG_LEVEL"] = "3"  # FATAL
@@ -367,7 +366,7 @@ class TripletNet:
                 loss_ml, hardest_ap_indices, hardest_an_indices = self.triplet_loss.call(*loss_inputs)
                 
                 # get selected/mined triplets
-                mined_triplets = utils.construct_mined_triplets(inputs=inputs, ap_indices=hardest_ap_indices, 
+                mined_triplets = construct_mined_triplets(inputs=inputs, ap_indices=hardest_ap_indices, 
                                                                     an_indices=hardest_an_indices)
 
                 tf.summary.scalar("loss_ml_train", loss_ml, step=step)
@@ -383,7 +382,7 @@ class TripletNet:
                     overall_loss = self.overall_similarity_loss.call(loss_ml=loss_ml, loss_sm=sim_att_loss)
                 else:
                     overall_loss = loss_ml
-                    selected_triplets = utils.construct_mined_triplets(inputs, hardest_ap_indices, hardest_an_indices)
+                    selected_triplets = construct_mined_triplets(inputs, hardest_ap_indices, hardest_an_indices)
                     # TODO: call method to draw images with heatmap
                     # all_triplet_att_images = add_heatmap_to_image(...)
 
@@ -446,7 +445,7 @@ class TripletNet:
 
                 # calc triplet loss
                 loss_ml, hardest_ap_indices, hardest_an_indices = self.triplet_loss.call(*loss_inputs)
-                self.mined_triplets = utils.construct_mined_triplets(inputs=images, ap_indices=hardest_ap_indices, 
+                self.mined_triplets = construct_mined_triplets(inputs=images, ap_indices=hardest_ap_indices, 
                                                                     an_indices=hardest_an_indices)
 
 
@@ -465,7 +464,7 @@ class TripletNet:
                     overall_loss = self.overall_similarity_loss.call(loss_ml=loss_ml, loss_sm=sim_att_loss)
                 else:
                     overall_loss = loss_ml
-                    selected_triplets = utils.construct_mined_triplets(inputs, hardest_ap_indices, hardest_an_indices)
+                    selected_triplets = tu.construct_mined_triplets(inputs, hardest_ap_indices, hardest_an_indices)
                     # TODO: call method to draw images with heatmap
                     # all_triplet_att_images = draw_images_with_simiarlity_maps(...)
 
